library ac_dart.ac_jwt.addon_session_token.test;

import 'package:atlassian_connect_jwt/src/addon_session_token.dart';
import 'package:dart_jwt/dart_jwt.dart';

//import 'package:crypto/crypto.dart';

import 'package:unittest/unittest.dart';

void main()  {
  final String addonSecret = '3ab90b11-d7bd-4097-958f-01b7ac4e985f';
  final String addonKey = 'my-addon';
  final String subject = 'fred';
  final String productHostKey = 'jira:ae390d29-31b2-4c12-a719-9df64e3e92b7';

  final DateTime expiry = DateTime.parse('2014-03-07 15:26:07.000');
  final DateTime issuedAt = DateTime.parse('2014-03-07 15:23:07.000');
  final JwaSignatureContext signatureContext = new JwaSignatureContext(addonSecret);

  final String jwtStr = r'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpYXQiOjEzOTQxNjYxODcsImV4cCI6MTM5NDE2NjM2NywiaXNzIjoibXktYWRkb24iLCJzdWIiOiJmcmVkIiwicHJvZHVjdEhvc3RLZXkiOiJqaXJhOmFlMzkwZDI5LTMxYjItNGMxMi1hNzE5LTlkZjY0ZTNlOTJiNyJ9.fDIpt5lbFpzoa_K7TvBmupzZcByP1Trvt4RB6-JOHSQ';

  final uriStr = "/hello-world?lic=none&tz=Australia%2FSydney&cp=%2Fjira&user_key=&loc=en-US&user_id=&xdm_e=http%3A%2F%2Fstorm%3A2990&xdm_c=channel-servlet-hello-world&xdm_p=1";
  final uri = Uri.parse(uriStr);
  final lenientClaimSetValidationContext = new JwtClaimSetValidationContext(
      expiryTolerance: const Duration(days: 365*1000));
  
  final JwtValidationContext validationContext = 
      new JwtValidationContext(signatureContext, lenientClaimSetValidationContext);
  
  group('[decode]', () {
    JsonWebToken jwt() => decodeAddonSessionToken(jwtStr);
    AddonSessionClaimSet claimSet() => jwt().claimSet;
    
    group('[claimset]', () {
      test('issuer parses', () {
        expect(claimSet().issuer, equals(addonKey));
      });
      test('subject parses', () {
        expect(claimSet().subject, equals(subject));
      });
      test('expiry parses', () {
        expect(claimSet().expiry, equals(expiry));
      });
      test('issuedAt parses', () {
        expect(claimSet().issuedAt, equals(issuedAt));
      });
      test('productHostKey parses', () {
        expect(claimSet().productHostKey, equals(productHostKey));
      });
    });
    skip_group('[signature]', () {
      test('validates successfully with correct shared secret', () {
        Set<ConstraintViolation> violations = jwt().validate(validationContext);
        expect(violations, isEmpty);        
      });
    });
  });
  
  group('[encode]', () {
    final claimSet = new AddonSessionClaimSet(
        addonKey,
        subject,
        expiry,
        issuedAt,
        productHostKey);
       
    JsonWebToken jwt() => new JsonWebToken.jws(claimSet, signatureContext);
    String encode() => jwt().encode();
    JsonWebToken parseEncoded() => decodeAddonSessionToken(encode(), validationContext: validationContext);
    AddonSessionClaimSet roundtripClaimSet() => parseEncoded().claimSet;
    
    
    group('[roundtrip]', () {
      test('issuer matches', () {
        expect(roundtripClaimSet().issuer, equals(addonKey));
      });
      test('subject matches', () {
        expect(roundtripClaimSet().subject, equals(subject));
      });
      test('expiry matches', () {
        expect(roundtripClaimSet().expiry, equals(expiry));
      });
      test('issuedAt matches', () {
        expect(roundtripClaimSet().issuedAt, equals(issuedAt));
      });
      test('productHostKey matches', () {
        expect(roundtripClaimSet().productHostKey, equals(productHostKey));
      });
    });
  });
  
  group('[validation]', () {

    final claimSetValidationContext = new JwtClaimSetValidationContext(
        expiryTolerance: const Duration(seconds: 30));

    AddonSessionClaimSet claimSet(int secondsBeforeNow) => 
        new AddonSessionClaimSet(
          addonKey,
          subject,
          new DateTime.now().subtract(new Duration(seconds: secondsBeforeNow)),
          issuedAt,
          productHostKey);
    
    Set<ConstraintViolation> violations(int secondsBeforeNow) => 
        claimSet(secondsBeforeNow)
          .validate(claimSetValidationContext);
    
    test('fails validation if more than tolerance past expiry', () {
      expect(violations(31), isNot(isEmpty));
    });
  
    test('passes validation if no more than tolerance past expiry', () {
      expect(violations(30), isEmpty);
    });
    
  });
  
}

